package com.usky.demo.service.config.mqtt;

import lombok.Data;
import org.eclipse.paho.client.mqttv3.MqttConnectOptions;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.integration.mqtt.core.DefaultMqttPahoClientFactory;
import org.springframework.integration.mqtt.core.MqttPahoClientFactory;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

@Data
@Component
public class MqttBaseConfig {

	@Value("${mqtt.username}")
	private String username;

	@Value("${mqtt.password}")
	private String password;

	@Value("${mqtt.url}")
	private String hostUrl;

	@Value("${mqtt.keep-alive-interval}")
	//心跳间隔
	private int keepAliveInterval;
	@Value("${mqtt.completionTimeout}")
	//心跳间隔
	private int completionTimeout;
	/**
	 * 连接工厂配置
	 */
	private static final String CLIENT_FACTORY = "mqttClientFactory";

	@Bean(name = CLIENT_FACTORY)
	public MqttPahoClientFactory mqttClientFactory() {
		List<String> urlList = new ArrayList<>();
		urlList.add(this.getHostUrl());
		String[] serverURIs = new String[urlList.size()];
		urlList.toArray(serverURIs);
		DefaultMqttPahoClientFactory factory = new DefaultMqttPahoClientFactory();
		MqttConnectOptions options = new MqttConnectOptions();
		options.setServerURIs(serverURIs);
		options.setUserName(this.getUsername());
		options.setPassword(this.getPassword().toCharArray());
		factory.setConnectionOptions(options);
		return factory;
	}

}